using System.Linq;
using Microsoft.WindowsAzure.Diagnostics;
using Microsoft.WindowsAzure.ServiceRuntime;
using Microsoft.WindowsAzure;
using System.Web.UI.WebControls;
using System.Diagnostics;
using AzureHelperLib;

namespace WebRole1
{
    public class WebRole : RoleEntryPoint
    {
        public override bool OnStart()
        {


            DiagnosticMonitorConfiguration config = DiagnosticMonitor.GetDefaultInitialConfiguration();
            config.DiagnosticInfrastructureLogs.ScheduledTransferPeriod = System.TimeSpan.FromMinutes(1);
            config.DiagnosticInfrastructureLogs.ScheduledTransferLogLevelFilter = LogLevel.Error;

            //assign scheduled transfer interval for user's Windows Azure Logs to 1 minute; NOT good for production
            config.Logs.ScheduledTransferPeriod = System.TimeSpan.FromMinutes(1);

            //create performance counter
            var performanceConfiguration = new PerformanceCounterConfiguration();
            performanceConfiguration.CounterSpecifier = @"\Processor(_Total)\% Processor Time";
            performanceConfiguration.SampleRate = System.TimeSpan.FromSeconds(1.0);
            
            //add counter to the configuration 
            config.PerformanceCounters.DataSources.Add(performanceConfiguration);
            config.PerformanceCounters.ScheduledTransferPeriod = System.TimeSpan.FromSeconds(20.0);

            DiagnosticMonitor.Start("DiagnosticsConnectionString", config);

            RoleEnvironment.Changing += RoleEnvironmentChanging;

            CloudStorageAccount.SetConfigurationSettingPublisher((configName, configSetter) =>
            {
                configSetter(RoleEnvironment.GetConfigurationSettingValue(configName));
                RoleEnvironment.Changed += (sender, arg) =>
                {
                    if (arg.Changes.OfType<RoleEnvironmentConfigurationSettingChange>()
                        .Any((change) => (change.ConfigurationSettingName == configName)))
                    {
                        if (!configSetter(RoleEnvironment.GetConfigurationSettingValue(configName)))
                        {
                            RoleEnvironment.RequestRecycle();
                        }
                    }
                };
            });

            return base.OnStart();
        }

        private void RoleEnvironmentChanging(object sender, RoleEnvironmentChangingEventArgs e)
        {
            if (e.Changes.Any(change => change is RoleEnvironmentConfigurationSettingChange))
            {
                e.Cancel = true;
            }
        }

    }
}
