﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading;
using Microsoft.WindowsAzure.Diagnostics;
using Microsoft.WindowsAzure.ServiceRuntime;
using Microsoft.WindowsAzure;
using AzureHelperLib;

namespace WorkerRole1
{
    public class WorkerRole : RoleEntryPoint
    {

        public override void Run()
        {
            Trace.WriteLine("WorkerRole1 entry point called", "Information");

            System.Timers.Timer checkCPUTimer = new System.Timers.Timer();
            checkCPUTimer.Interval = 300000;  //use 300000.0 for 5 minutes
            checkCPUTimer.Elapsed += new System.Timers.ElapsedEventHandler(checkCPUTimer_Elapsed);
            checkCPUTimer.Start();

            while (true)
            {
                Thread.Sleep(10000);
                Trace.WriteLine("Working", "Information");
            }

        }

        void checkCPUTimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            TestPerfData();
        }



        public override bool OnStart()
        {
            // Set the maximum number of concurrent connections 
            ServicePointManager.DefaultConnectionLimit = 12;

            DiagnosticMonitor.Start("DiagnosticsConnectionString");


            // For information on handling configuration changes
            // see the MSDN topic at http://go.microsoft.com/fwlink/?LinkId=166357.
            RoleEnvironment.Changing += RoleEnvironmentChanging;
            RoleEnvironment.Changing += RoleEnvironmentChanging;

            CloudStorageAccount.SetConfigurationSettingPublisher((configName, configSetter) =>
            {
                configSetter(RoleEnvironment.GetConfigurationSettingValue(configName));
                RoleEnvironment.Changed += (sender, arg) =>
                {
                    if (arg.Changes.OfType<RoleEnvironmentConfigurationSettingChange>()
                        .Any((change) => (change.ConfigurationSettingName == configName)))
                    {
                        if (!configSetter(RoleEnvironment.GetConfigurationSettingValue(configName)))
                        {
                            RoleEnvironment.RequestRecycle();
                        }
                    }
                };
            });
            return base.OnStart();
        }

        private void RoleEnvironmentChanging(object sender, RoleEnvironmentChangingEventArgs e)
        {
            // If a configuration setting is changing
            if (e.Changes.Any(change => change is RoleEnvironmentConfigurationSettingChange))
            {
                // Set e.Cancel to true to restart this role instance
                e.Cancel = true;
            }
        }

        public void TestPerfData()
        {
            Trace.TraceInformation("Starting TestPerfData");
            try
            {
                var account = CloudStorageAccount.FromConfigurationSetting("DataConnectionString");
                var context = new PerformanceDataContext(account.TableEndpoint.ToString(), account.Credentials);
                var data = context.PerfData;

                System.Collections.Generic.List<PerformanceData> selectedData = (from d in data
                                                                                 where d.CounterName == @"\Processor(_Total)\% Processor Time"
                                                                                 select d).ToList<PerformanceData>();

                double AvgCPU = (from d in selectedData
                             where d.CounterName == @"\Processor(_Total)\% Processor Time"
                             select d.CounterValue).Average();
                
                Trace.WriteLine("Average CPU == " + AvgCPU.ToString());
                //test threshold
                if (AvgCPU > 85.0)
                {
                    //increase instances
                }
                else if (AvgCPU < 85.0)
                {
                    Trace.TraceInformation("in the AvgCPU < 85 test.");

                    string deploymentInfo = AzureRESTMgmtHelper.GetDeploymentInfo();
                    string svcconfig = AzureRESTMgmtHelper.GetServiceConfig(deploymentInfo);
                    int InstanceCount = System.Convert.ToInt32(AzureRESTMgmtHelper.GetInstanceCount(svcconfig, "WebRole1"));

                    if (InstanceCount > 1)
                    {
                        InstanceCount--;
                        string UpdatedSvcConfig = AzureRESTMgmtHelper.ChangeInstanceCount(svcconfig, "WebRole1", InstanceCount.ToString());

                        AzureRESTMgmtHelper.ChangeConfigFile(UpdatedSvcConfig);
                    }
                }

            }
            catch (System.Exception ex)
            {
                Trace.WriteLine("Worker Role Error: " + ex.Message);
            }


        }
    }
}
